var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/index.tsx
import * as React from "react";
import convert from "react-from-dom";

// src/config.ts
var CACHE_NAME = "react-inlinesvg";
var CACHE_MAX_RETRIES = 10;
var STATUS = {
  IDLE: "idle",
  LOADING: "loading",
  LOADED: "loaded",
  FAILED: "failed",
  READY: "ready",
  UNSUPPORTED: "unsupported"
};

// src/helpers.ts
function canUseDOM() {
  return !!(typeof window !== "undefined" && window.document && window.document.createElement);
}
function isSupportedEnvironment() {
  return supportsInlineSVG() && typeof window !== "undefined" && window !== null;
}
async function request(url, options) {
  const response = await fetch(url, options);
  const contentType = response.headers.get("content-type");
  const [fileType] = (contentType || "").split(/ ?; ?/);
  if (response.status > 299) {
    throw new Error("Not found");
  }
  if (!["image/svg+xml", "text/plain"].some((d) => fileType.includes(d))) {
    throw new Error(`Content type isn't valid: ${fileType}`);
  }
  return response.text();
}
function sleep(seconds = 1) {
  return new Promise((resolve) => {
    setTimeout(resolve, seconds * 1e3);
  });
}
function supportsInlineSVG() {
  if (!document) {
    return false;
  }
  const div = document.createElement("div");
  div.innerHTML = "<svg />";
  const svg = div.firstChild;
  return !!svg && svg.namespaceURI === "http://www.w3.org/2000/svg";
}
function randomCharacter(character) {
  return character[Math.floor(Math.random() * character.length)];
}
function randomString(length) {
  const letters = "abcdefghijklmnopqrstuvwxyz";
  const numbers = "1234567890";
  const charset = `${letters}${letters.toUpperCase()}${numbers}`;
  let R = "";
  for (let index = 0; index < length; index++) {
    R += randomCharacter(charset);
  }
  return R;
}
function omit(input, ...filter) {
  const output = {};
  for (const key in input) {
    if ({}.hasOwnProperty.call(input, key)) {
      if (!filter.includes(key)) {
        output[key] = input[key];
      }
    }
  }
  return output;
}

// src/cache.ts
var CacheStore = class {
  constructor() {
    __publicField(this, "cacheApi");
    __publicField(this, "cacheStore");
    __publicField(this, "subscribers", []);
    __publicField(this, "isReady", false);
    this.cacheStore = /* @__PURE__ */ new Map();
    let cacheName = CACHE_NAME;
    let usePersistentCache = false;
    if (canUseDOM()) {
      cacheName = window.REACT_INLINESVG_CACHE_NAME ?? CACHE_NAME;
      usePersistentCache = !!window.REACT_INLINESVG_PERSISTENT_CACHE;
    }
    if (usePersistentCache) {
      caches.open(cacheName).then((cache) => {
        this.cacheApi = cache;
        this.isReady = true;
        this.subscribers.forEach((callback) => callback());
      });
    } else {
      this.isReady = true;
    }
  }
  onReady(callback) {
    if (this.isReady) {
      callback();
    } else {
      this.subscribers.push(callback);
    }
  }
  async get(url, fetchOptions) {
    await (this.cacheApi ? this.fetchAndAddToPersistentCache(url, fetchOptions) : this.fetchAndAddToInternalCache(url, fetchOptions));
    return this.cacheStore.get(url)?.content ?? "";
  }
  set(url, data) {
    this.cacheStore.set(url, data);
  }
  isCached(url) {
    return this.cacheStore.get(url)?.status === STATUS.LOADED;
  }
  async fetchAndAddToInternalCache(url, fetchOptions) {
    const cache = this.cacheStore.get(url);
    if (cache?.status === STATUS.LOADING) {
      await this.handleLoading(url, async () => {
        this.cacheStore.set(url, { content: "", status: STATUS.IDLE });
        await this.fetchAndAddToInternalCache(url, fetchOptions);
      });
      return;
    }
    if (!cache?.content) {
      this.cacheStore.set(url, { content: "", status: STATUS.LOADING });
      try {
        const content = await request(url, fetchOptions);
        this.cacheStore.set(url, { content, status: STATUS.LOADED });
      } catch (error) {
        this.cacheStore.set(url, { content: "", status: STATUS.FAILED });
        throw error;
      }
    }
  }
  async fetchAndAddToPersistentCache(url, fetchOptions) {
    const cache = this.cacheStore.get(url);
    if (cache?.status === STATUS.LOADED) {
      return;
    }
    if (cache?.status === STATUS.LOADING) {
      await this.handleLoading(url, async () => {
        this.cacheStore.set(url, { content: "", status: STATUS.IDLE });
        await this.fetchAndAddToPersistentCache(url, fetchOptions);
      });
      return;
    }
    this.cacheStore.set(url, { content: "", status: STATUS.LOADING });
    const data = await this.cacheApi?.match(url);
    if (data) {
      const content = await data.text();
      this.cacheStore.set(url, { content, status: STATUS.LOADED });
      return;
    }
    try {
      await this.cacheApi?.add(new Request(url, fetchOptions));
      const response = await this.cacheApi?.match(url);
      const content = await response?.text() ?? "";
      this.cacheStore.set(url, { content, status: STATUS.LOADED });
    } catch (error) {
      this.cacheStore.set(url, { content: "", status: STATUS.FAILED });
      throw error;
    }
  }
  async handleLoading(url, callback) {
    let retryCount = 0;
    while (this.cacheStore.get(url)?.status === STATUS.LOADING && retryCount < CACHE_MAX_RETRIES) {
      await sleep(0.1);
      retryCount += 1;
    }
    if (retryCount >= CACHE_MAX_RETRIES) {
      await callback();
    }
  }
  keys() {
    return [...this.cacheStore.keys()];
  }
  data() {
    return [...this.cacheStore.entries()].map(([key, value]) => ({ [key]: value }));
  }
  async delete(url) {
    if (this.cacheApi) {
      await this.cacheApi.delete(url);
    }
    this.cacheStore.delete(url);
  }
  async clear() {
    if (this.cacheApi) {
      const keys = await this.cacheApi.keys();
      for (const key of keys) {
        await this.cacheApi.delete(key);
      }
    }
    this.cacheStore.clear();
  }
};

// src/index.tsx
import { jsx } from "react/jsx-runtime";
var cacheStore;
var ReactInlineSVG = class extends React.PureComponent {
  constructor(props) {
    super(props);
    __publicField(this, "hash");
    __publicField(this, "isActive", false);
    __publicField(this, "isInitialized", false);
    __publicField(this, "fetchContent", async () => {
      const { fetchOptions, src } = this.props;
      const content = await request(src, fetchOptions);
      this.handleLoad(content);
    });
    __publicField(this, "handleError", (error) => {
      const { onError } = this.props;
      const status = error.message === "Browser does not support SVG" ? STATUS.UNSUPPORTED : STATUS.FAILED;
      if (this.isActive) {
        this.setState({ status }, () => {
          if (typeof onError === "function") {
            onError(error);
          }
        });
      }
    });
    __publicField(this, "handleLoad", (content, hasCache = false) => {
      if (this.isActive) {
        this.setState(
          {
            content,
            isCached: hasCache,
            status: STATUS.LOADED
          },
          this.getElement
        );
      }
    });
    this.state = {
      content: "",
      element: null,
      isCached: !!props.cacheRequests && cacheStore.isCached(props.src),
      status: STATUS.IDLE
    };
    this.hash = props.uniqueHash || randomString(8);
  }
  componentDidMount() {
    this.isActive = true;
    if (!canUseDOM() || this.isInitialized) {
      return;
    }
    const { status } = this.state;
    const { src } = this.props;
    try {
      if (status === STATUS.IDLE) {
        if (!isSupportedEnvironment()) {
          throw new Error("Browser does not support SVG");
        }
        if (!src) {
          throw new Error("Missing src");
        }
        this.load();
      }
    } catch (error) {
      this.handleError(error);
    }
    this.isInitialized = true;
  }
  componentDidUpdate(previousProps, previousState) {
    if (!canUseDOM()) {
      return;
    }
    const { isCached, status } = this.state;
    const { onLoad, src } = this.props;
    if (previousState.status !== STATUS.READY && status === STATUS.READY) {
      if (onLoad) {
        onLoad(src, isCached);
      }
    }
    if (previousProps.src !== src) {
      if (!src) {
        this.handleError(new Error("Missing src"));
        return;
      }
      this.load();
    }
  }
  componentWillUnmount() {
    this.isActive = false;
  }
  getElement() {
    try {
      const node = this.getNode();
      const element = convert(node);
      if (!element || !React.isValidElement(element)) {
        throw new Error("Could not convert the src to a React element");
      }
      this.setState({
        element,
        status: STATUS.READY
      });
    } catch (error) {
      this.handleError(new Error(error.message));
    }
  }
  getNode() {
    const { description, title } = this.props;
    try {
      const svgText = this.processSVG();
      const node = convert(svgText, { nodeOnly: true });
      if (!node || !(node instanceof SVGSVGElement)) {
        throw new Error("Could not convert the src to a DOM Node");
      }
      const svg = this.updateSVGAttributes(node);
      if (description) {
        const originalDesc = svg.querySelector("desc");
        if (originalDesc && originalDesc.parentNode) {
          originalDesc.parentNode.removeChild(originalDesc);
        }
        const descElement = document.createElementNS("http://www.w3.org/2000/svg", "desc");
        descElement.innerHTML = description;
        svg.prepend(descElement);
      }
      if (typeof title !== "undefined") {
        const originalTitle = svg.querySelector("title");
        if (originalTitle && originalTitle.parentNode) {
          originalTitle.parentNode.removeChild(originalTitle);
        }
        if (title) {
          const titleElement = document.createElementNS("http://www.w3.org/2000/svg", "title");
          titleElement.innerHTML = title;
          svg.prepend(titleElement);
        }
      }
      return svg;
    } catch (error) {
      return this.handleError(error);
    }
  }
  load() {
    if (this.isActive) {
      this.setState(
        {
          content: "",
          element: null,
          isCached: false,
          status: STATUS.LOADING
        },
        async () => {
          const { cacheRequests, fetchOptions, src } = this.props;
          const dataURI = src.match(/^data:image\/svg[^,]*?(;base64)?,(.*)/u);
          let inlineSrc;
          if (dataURI) {
            inlineSrc = dataURI[1] ? window.atob(dataURI[2]) : decodeURIComponent(dataURI[2]);
          } else if (src.includes("<svg")) {
            inlineSrc = src;
          }
          if (inlineSrc) {
            this.handleLoad(inlineSrc);
            return;
          }
          try {
            if (cacheRequests) {
              const content = await cacheStore.get(src, fetchOptions);
              this.handleLoad(content, true);
            } else {
              await this.fetchContent();
            }
          } catch (error) {
            this.handleError(error);
          }
        }
      );
    }
  }
  processSVG() {
    const { content } = this.state;
    const { preProcessor } = this.props;
    if (preProcessor) {
      return preProcessor(content);
    }
    return content;
  }
  updateSVGAttributes(node) {
    const { baseURL = "", uniquifyIDs } = this.props;
    const replaceableAttributes = ["id", "href", "xlink:href", "xlink:role", "xlink:arcrole"];
    const linkAttributes = ["href", "xlink:href"];
    const isDataValue = (name, value) => linkAttributes.includes(name) && (value ? !value.includes("#") : false);
    if (!uniquifyIDs) {
      return node;
    }
    [...node.children].forEach((d) => {
      if (d.attributes && d.attributes.length) {
        const attributes = Object.values(d.attributes).map((a) => {
          const attribute = a;
          const match = a.value.match(/url\((.*?)\)/);
          if (match && match[1]) {
            attribute.value = a.value.replace(match[0], `url(${baseURL}${match[1]}__${this.hash})`);
          }
          return attribute;
        });
        replaceableAttributes.forEach((r) => {
          const attribute = attributes.find((a) => a.name === r);
          if (attribute && !isDataValue(r, attribute.value)) {
            attribute.value = `${attribute.value}__${this.hash}`;
          }
        });
      }
      if (d.children.length) {
        return this.updateSVGAttributes(d);
      }
      return d;
    });
    return node;
  }
  render() {
    const { element, status } = this.state;
    const { children = null, innerRef, loader = null } = this.props;
    const elementProps = omit(
      this.props,
      "baseURL",
      "cacheRequests",
      "children",
      "description",
      "fetchOptions",
      "innerRef",
      "loader",
      "onError",
      "onLoad",
      "preProcessor",
      "src",
      "title",
      "uniqueHash",
      "uniquifyIDs"
    );
    if (!canUseDOM()) {
      return loader;
    }
    if (element) {
      return React.cloneElement(element, { ref: innerRef, ...elementProps });
    }
    if ([STATUS.UNSUPPORTED, STATUS.FAILED].includes(status)) {
      return children;
    }
    return loader;
  }
};
__publicField(ReactInlineSVG, "defaultProps", {
  cacheRequests: true,
  uniquifyIDs: false
});
function InlineSVG(props) {
  if (!cacheStore) {
    cacheStore = new CacheStore();
  }
  const { loader } = props;
  const hasCallback = React.useRef(false);
  const [isReady, setReady] = React.useState(cacheStore.isReady);
  React.useEffect(() => {
    if (!hasCallback.current) {
      cacheStore.onReady(() => {
        setReady(true);
      });
      hasCallback.current = true;
    }
  }, []);
  if (!isReady) {
    return loader;
  }
  return /* @__PURE__ */ jsx(ReactInlineSVG, { ...props });
}
export {
  cacheStore,
  InlineSVG as default
};
//# sourceMappingURL=index.mjs.map